/**
 * Copyright (c) 2002-2013 "Neo Technology,"
 * Network Engine for Objects in Lund AB [http://neotechnology.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.neo4j.cypher.internal.parser.experimental.ast

import org.neo4j.cypher.internal.parser.experimental._
import org.neo4j.cypher.internal.symbols._
import org.neo4j.cypher.internal.commands.{expressions => commandexpressions}
import org.neo4j.cypher.internal.commands.expressions.{Expression => CommandExpression}
import org.neo4j.cypher.internal.parser.experimental.ast.Expression.SemanticContext

case class ShortestPathExpression(pattern: ShortestPath) extends Expression with SimpleTypedExpression {
  def token = pattern.token
  protected def possibleTypes = Set(CollectionType(PathType()))

  override def semanticCheck(ctx: SemanticContext) = {
    pattern.semanticCheck(Pattern.SemanticContext.Expression) then super.semanticCheck(ctx)
  }

  def toCommand: CommandExpression = commandexpressions.ShortestPathExpression(pattern.toLegacyPatterns(None).head)
}
